
import com.sun.j3d.utils.geometry.*;
import javax.vecmath.*;
import javax.media.j3d.*;




// **********************************************
// Marker
//          class to represent a single marker
//                with its own path interpolator
//              caller has to add locatorTG and posPathInt
//              children to appropriate parent.
// **********************************************

public class Marker {
  
    // instance fields
    //   these to be added to parentTG
    public TransformGroup locatorTG;
    public PositionPathInterpolator posPathInt;
    
    public Point3f [] locationArray;
    private Point3f neutralLocation;
    private float[] kn;
    
    private Vector3f v3f;
    private Transform3D t3d;
    
    // constructors for the class Marker
    public Marker(){
    }
    
    public Marker( float markersize,
		   Appearance colorApp,
		   Transform3D localAxis,
		   float [] knots,
		   Point3f [] locs,
		   Point3f neutralLoc,
		   Alpha alpha,
		   BoundingSphere bounds,
		   TransformGroup parentTG
		   ) {
	// sets up the interpolator, but disables it and 
	// poses figure in neutral position instead
	
	locationArray = locs;
	neutralLocation = neutralLoc;
	kn = knots;
	
	Sphere marker = new Sphere(markersize, colorApp);
	v3f = new Vector3f(neutralLocation);
	t3d = new Transform3D();
	t3d.setTranslation(v3f);
	locatorTG = new TransformGroup(t3d);
	BranchGroup mBG = new BranchGroup();
	mBG.setCapability(BranchGroup.ALLOW_DETACH);
	locatorTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	posPathInt = 
	    new PositionPathInterpolator(alpha, locatorTG, localAxis,
					 kn, locationArray);
	posPathInt.setEnable(false);
	posPathInt.setSchedulingBounds(bounds);
	locatorTG.addChild(marker);
	mBG.addChild(locatorTG);
	mBG.addChild(posPathInt);
	parentTG.addChild(mBG);
    }
    
    
    // *** Utility Methods for Marker Class
    // disable the path interpolator
    public void stopper() {
	if (posPathInt.getEnable())
	    posPathInt.setEnable(false);
    }
    
    // enable the path interpolator
    public void starter() {
	if (!posPathInt.getEnable())
	    posPathInt.setEnable(true);
    }
    
    // stuff a single value in the transform for static display
    public void setLoc(int j) {
	if (posPathInt.getEnable()) 
	    posPathInt.setEnable(false);
	if (j > -1) 
	    v3f.set(locationArray[j]);
	else
	    v3f.set(neutralLocation);
	
	t3d.setTranslation(v3f);
	locatorTG.setTransform(t3d);
    }	    
    
}   // end of Marker class













