import java.awt.*;
import java.util.*;

// ***********************************************
//  a custom component from Java Examples in a Nutshell
//  
// implemented by M. Geroch
// November, 2000
// *******************************************

public class MultiLineLabel extends Canvas {
    protected String label;
    protected int margin_width, margin_height, alignment;
    public static final int LEFT = 0, CENTER = 1, RIGHT = 2;
    protected int num_lines; 
    protected String[] lines;
    protected int[] line_widths;
    protected int max_width, line_height;
    protected int line_ascent;
    protected boolean measured = false;

    // ====================================
    // five constructors
    // ------------------------------------
    public MultiLineLabel(String label, int margin_width,
			  int margin_height, int alignment) {
	this.label = label;
	this.margin_width = margin_width;
	this.margin_height = margin_height;
	this.alignment = alignment;
	newLabel();
    }
    public MultiLineLabel(String label, int margin_width, int margin_height) {
	this(label, margin_width, margin_height, LEFT);
    }
    public MultiLineLabel(String label, int alignment) {
	this(label, 10, 10, alignment);
    }
    public MultiLineLabel(String label) {
	this(label, 10, 10, LEFT);
    }
    public MultiLineLabel() { this(""); }

    // =================================================
    // methods to set and query the components
    // --------------------------------------------
    public void setLabel(String label) {
	this.label = label;
	newLabel();
	measured = false;
	repaint();
    }

    public void setFont(Font f) {
	super.setFont(f);
	measured = false;
	repaint();
    }
    public void setForeground( Color c) {
	super.setForeground(c);
	repaint();
    }
    public void setAlignment(int a) { alignment = a; repaint(); }
    public void setMarginWidth( int mw) { margin_width = mw; repaint(); }
    public String getLabel() { return label; }
    public int getAlignment() { return alignment; }
    public int getMarginWidth() { return margin_width; }
    public int getMarginHeight() { return margin_height; }

    // ==========================================
    // use getPreferredSize() and getMinimumSize()
    
    // =====================================
    public void paint(Graphics g) {
	int x, y;
	Dimension size = this.getSize();
	if (!measured) measure();
	y = line_ascent + (size.height - num_lines*line_height)/2;
	for (int i = 0; i < num_lines; ++i, y += line_height) {
	    switch(alignment) {
	    default:
	    case LEFT:  x = margin_width; break;
	    case CENTER: x = (size.width - line_widths[i])/2; break;
	    case RIGHT: x = size.width - margin_width - line_widths[i]; break;
	    }
	    g.drawString(lines[i], x, y);
	}
    }

    protected synchronized void newLabel() {
	StringTokenizer t = new StringTokenizer(label, "\n");
	num_lines = t.countTokens();
	lines = new String[num_lines];
	line_widths = new int[num_lines];
	for(int i = 0; i < num_lines; ++i)
	    lines[i] = t.nextToken();
    }

    protected synchronized void measure() {
	//FontMetrics fm = this.getToolkit().getFontMetrics(this.getFont());
	line_height = 17;//fm.getHeight();

	//System.out.println(line_height);
	line_ascent = 13;//fm.getAscent();

	//System.out.println(line_ascent);


	max_width = 267;

	line_widths[0] = 267;

	line_widths[1] = 223;

	line_widths[2] = 168;

	line_widths[3] = 123;



	//System.out.println("widths:");
	/*for (int i = 0; i < num_lines; ++i) {
	    line_widths[i] = fm.stringWidth(lines[i]);

		System.out.println(line_widths[i]);
	    if (line_widths[i] > max_width) max_width = line_widths[i];
	}*/

	
	measured = true;
    }
}

